/***************************************************************************//**
 *   @file   AD511X.c
 *   @brief  Implementation of AD511X Driver.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 782
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "AD511X.h"			    // AD511X definitions.
#include "Communication.h"		// Communication definitions.

/*****************************************************************************/
/************************ Variables Definitions ******************************/
/*****************************************************************************/
char  bitNr  = 0;
float rW     = 70;
float rBS    = 45;
float rAB    = 10000;
float resTol = 0;


/**************************************************************************//**
 * @brief Initializes the communication with the device.
 *
 * @param deviceModel - The device in use,
 *                      Example: AD5110
 *                               AD5112
 *                               AD5114
 * @param endToEndRes - End to end resistance of the device.
 *                      Example: AD511X_5_KOHM  -  5 KOhm
 *                               AD511X_10_KOHM - 10 KOhm
 *                               AD511X_80_KOHM - 80 KOhm
 *
 * @return status - Result of the initialization procedure.
 *					Example: 0x0 - I2C peripheral was not initialized.
 *				  			 0x1 - I2C peripheral is initialized.
*******************************************************************************/
unsigned char AD511X_Init(char deviceModel, long endToEndRes)
{  
    char status = 0;
    
    status = I2C_Init(100000);
    /* Store the ideal end-to-end resistance of the device. */
    rAB = endToEndRes;
    /* Select the number of bits of the selected device. */
    switch(deviceModel)
    {
        case AD5110:
            bitNr = 7; // 128 steps
            break;
        case AD5112:
            bitNr = 6; // 64 steps
            break;
        case AD5114:
            bitNr = 5; // 32 steps
            break;            
    }
    /* Read the resistor tolerance from the device. */
    resTol = AD511X_GetTolerance();
    /* Calculate the actual end-to-end resistance.  */
    rAB *= (1 + (resTol / 100.0));
    
    return status;    
}

/**************************************************************************//**
 * @brief Sends a command to the device.
 *
 * @param command - Commands that facilitate easy manipulation of RDAC 
 *                  wiper settings.
 *                  Example: AD511X_NOP_CMD
 *                           AD511X_WR_RDAC_TO_EEPROM_CMD
 *                           AD511X_WR_SERIAL_TO_EEPROM_CMD
 *                           AD511X_SOFT_SHUTDOWN_CMD
 *                           AD511X_SOFT_RESET_CMD
 *                           AD511X_RD_RDAC_CMD
 *                           AD511X_RD_EEPROM
 *
 * @param data - Value to write to the device. Useful for writing data to RDAC
 *               register and also can be used in conjunction with other 
 *               commands to fulfill other functions.
 *               Example: 
 *                AD511X_DATA_TOP_SCALE     - used with Write to RDAC command
 *                AD511X_DATA_BOTTOM_SCALE  - used with Write to RDAC command
 *                AD511X_DATA_SHUTDOWN_OFF  - used with Shutdown command
 *                AD511X_DATA_SHUTDOWN_ON   - used with Shutdown command
 *                AD511X_DATA_WIPER_POS     - used with Read EEPROM command
 *                AD511X_DATA_RESISTOR_TOL  - used with Read EEPROM command
 *
 * @return None.
*******************************************************************************/
void AD511X_SendCommand(unsigned char command, unsigned char data)
{
    unsigned char i2cWord[2] = {0x00, 0x00};
    
    i2cWord[0] = command;
    i2cWord[1] = data;
                 
    I2C_Write(AD511X_SLAVE_ADDRESS, i2cWord, 2, 1);
}

/**************************************************************************//**
 * @brief Powers on/off the device.
 *
 * @param pwr - Power option.
 *              Example: 0 - shutdown the device;
 *                       1 - power-on the device.
 *
 * @return None.
*******************************************************************************/
void AD511X_Power(char pwr)
{
    char shutdownBit = 0;
    
    shutdownBit = (pwr == 1) ? AD511X_DATA_SHUTDOWN_OFF : 
                               AD511X_DATA_SHUTDOWN_ON ;
    AD511X_SendCommand(AD511X_SOFT_SHUTDOWN_CMD, shutdownBit);
}

/**************************************************************************//**
 * @brief Reads data from the device.
 *
 * @param readLocation - Selects the RDAC location or EEPROM location.
 *                       Example: AD511X_RD_RDAC_CMD   - RDAC location;
 *                                AD511X_RD_EEPROM_CMD - EEPROM location.
 * @param content - Selects a register from EEPROM.
 *                  Example: AD511X_DATA_WIPER_POS - the saved wiper position;
 *                           AD511X_DATA_RESISTOR_TOL - resistor tolerance;
 *                           0, when readLocation = AD511X_RD_RDAC_CMD.
 *
 * @return 8-bit data from the device.
*******************************************************************************/
unsigned char AD511X_ReadData(char readLocation, char content)
{
    unsigned char i2cWord[2] = {0x00, 0x00};
    
    AD511X_SendCommand(readLocation, content);
    I2C_Read(AD511X_SLAVE_ADDRESS, i2cWord, 2, 1);
    /* The data position differs from one version of device to another. */
    if(content != AD511X_DATA_RESISTOR_TOL)
    {
        i2cWord[0] >>= 7 - (bitNr);
    }
    
    return i2cWord[0];
}

/**************************************************************************//**
 * @brief Performs an Acknowledge Polling.
 *
 * @return None.
*******************************************************************************/
void AD511X_WaitForDevice(void)
{
    unsigned char i2cWord        = 0;
    unsigned char i2cWriteResult = 0;
    
    i2cWriteResult = I2C_Write(AD511X_SLAVE_ADDRESS, &i2cWord, 0, 1);
    while(i2cWriteResult == 0xFF)
    {
        i2cWriteResult = I2C_Write(AD511X_SLAVE_ADDRESS, &i2cWord, 0, 1);
    }   
}

/**************************************************************************//**
 * @brief Reads the resistance tolerance that is stored in the internal memory 
 *        during factory testing.
 *
 * @return Resistance tolerance represented as a float value.
*******************************************************************************/
float AD511X_GetTolerance(void)
{
    unsigned char readByte  = 0;
    float         tolerance = 0;    
    
    readByte = AD511X_ReadData(AD511X_RD_EEPROM_CMD, AD511X_DATA_RESISTOR_TOL);
    tolerance = (readByte & 0x78) >> 3;
    tolerance += (float)(readByte & 0x07) / 8.0; 
    if(readByte & 0x80)
    {
        tolerance *= -1;
    }
    
    return tolerance;
}

/**************************************************************************//**
 * @brief Sets the output resistance of the device.
 *
 * @param outRes - Output resistance value.
 *
 * @return The actual resistance that the device can output.
*******************************************************************************/
float AD511X_SetResistance(float outRes)
{
    unsigned char rawData   = 0;
    float         actualRes = 0;
    
    /* Set the output resistance to 45 Ohm(Bottom scale). */
    if(outRes < 70)
    {
        rawData = 0xFF;
    }
    else
    {
        rawData = (unsigned char)((outRes - rW) * (1 << bitNr) / rAB);
    }
    /* Values greater that 0x80 are not useful(except 0xFF). */
    if((rawData > 0x80) && (rawData != 0xFF))
    {
        rawData = 0x80;
    }
    /* Write raw data to RDAC. */
    AD511X_SendCommand(AD511X_WR_SERIAL_TO_RDAC_CMD, rawData << (7 - bitNr));
    if(rawData == 0xFF)
    {
        actualRes = rBS;
    }
    else
    {
        actualRes = ((float)rawData * rAB) / (1 << bitNr) + rW;
    }
    
    return actualRes;
}
